#!/usr/bin/python3

import RPi.GPIO as GPIO
from time import sleep
import sys
import os
import socket

# Global variables
CONTROL_IP      = '0.0.0.0'
CONTROL_PORT    = 10001
MOTOR_VALUE_MIN = 0
MOTOR_VALUE_MAX = 100
DEBUG = False

# GPIO settings
GPIO.setmode(GPIO.BCM)              # GPIO numbering
GPIO.setwarnings(False)             # enable warning from GPIO
AN1 = 12                            # set power pin for left motor
DIG1 = 26                           # set direction pin for left motor
AN2 = 13                            # set power pin for right motor
DIG2 = 24                           # set direction pin for right motor
GPIO.setup(AN1, GPIO.OUT)           # set power pin AN1 as output
GPIO.setup(AN2, GPIO.OUT)           # set power pin AN2 as output
GPIO.setup(DIG1, GPIO.OUT)          # set direction pin DIG1 as output
GPIO.setup(DIG2, GPIO.OUT)          # set direction pin DIG2 as output
p1 = GPIO.PWM(AN1, MOTOR_VALUE_MAX) # init power pin p1
p2 = GPIO.PWM(AN2, MOTOR_VALUE_MAX) # init power pin p2

# Functions
def go_forward(speed):
    p1.start(speed)
    GPIO.output(DIG1, GPIO.LOW)
    p2.start(speed)
    GPIO.output(DIG2, GPIO.LOW)

def go_reverse(speed):
    p1.start(speed)
    GPIO.output(DIG1, GPIO.HIGH)
    p2.start(speed)
    GPIO.output(DIG2, GPIO.HIGH)

def pivot_right(speed):
    p1.start(speed)
    GPIO.output(DIG1, GPIO.HIGH)
    p2.start(speed)
    GPIO.output(DIG2, GPIO.LOW)

def pivot_left(speed):
    p1.start(speed)
    GPIO.output(DIG1, GPIO.LOW)
    p2.start(speed)
    GPIO.output(DIG2, GPIO.HIGH)

def stop():
    p1.start(MOTOR_VALUE_MIN)
    p2.start(MOTOR_VALUE_MIN)

def start_control_server(ip,port):
    MAX_BYTES = 1024
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((ip, port))
    print('[MSG]> Server listening at {}'.format(sock.getsockname()))
    try:
        while True:
            # Receive data from client
            data, address = sock.recvfrom(MAX_BYTES)
            if data:
                # Decode client data
                data = data.decode('utf-8')
                # Get values from client data
                k_up,k_down,k_left,k_right = data.split(',')
                print("{},{},{},{}".format(k_up,k_down,k_left,k_right))
                # Apply values to motor control
                if (int(k_up)): go_forward(MOTOR_VALUE_MAX)
                elif (int(k_down)): go_reverse(MOTOR_VALUE_MAX)
                elif (int(k_left)): pivot_left(MOTOR_VALUE_MAX)
                elif (int(k_right)): pivot_right(MOTOR_VALUE_MAX)
                else: stop()
    finally:
        sock.close()
        GPIO.cleanup()

# Main
if __name__ == '__main__':
    os.system('clear')
    try:
        start_control_server(CONTROL_IP,CONTROL_PORT)
    except KeyboardInterrupt:
        print("Exit")
        sys.exit() 
